/*
	copytext.c
	Copies selected text (if any) from an XmText widget to the clipboard.

	mmore - A file/stream viewer for Motif/X
	Copyright (C) 2005   Matthew Holder (sixxgate@hotmail.com)

	derived from:
	mas - A file/stream viewer for X
	Copyright (C) 1998   Roberto Machorro (rmach@bigfoot.com)

	This program is free software; you can redistribute it and/or modify
	it under the terms of the GNU General Public License as published by
	the Free Software Foundation; either version 2 of the License, or
	(at your option) any later version.

	This program is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with this program; if not, write to the Free Software
	Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
*/

#include <Xm/Text.h>
#include <Xm/CutPaste.h>
#include "copytext.h"
#include "errdlg.h"

void copy(Widget w, XtPointer client_data, XmAnyCallbackStruct *call)
{
	XmString szAppName;
	XButtonEvent *p_event = (XButtonEvent *)call->event;
	Widget text = (Widget)client_data;

	char *szSelection = XmTextGetSelection(text);
	long item_id, data_id;
	int nRet, nLen;

	/* Make sure the there is selected text */
	if (szSelection == NULL) return;
	if (*szSelection == '\0')
	{
		XtFree(szSelection);
		return;
	}

	/* Locks the clipboard, for LessTif, I guess... NEdit did it. */
	if (XmClipboardLock(XtDisplay(text), XtWindow(text)) != ClipboardSuccess)
	{
		XtFree(szSelection);
		return;
	}

	/* Start the copy */
	szAppName = XmStringCreateLtoR ("MMore", XmSTRING_DEFAULT_CHARSET);
	nRet = XmClipboardStartCopy(XtDisplay(text), XtWindow(text),
			szAppName, p_event->time, text, NULL, &item_id);
	XmStringFree(szAppName);
	if (nRet != ClipboardSuccess)
	{
		XmClipboardUnlock(XtDisplay(text), XtWindow(text), False);
		XtFree(szSelection);
		return;
	}
	
	/* Perform the copy */
	nLen = strlen(szSelection);
	if (XmClipboardCopy(XtDisplay(text), XtWindow(text), item_id,
			       "STRING", szSelection, nLen,
			       0, NULL) != ClipboardSuccess)
	{
		XmClipboardEndCopy(XtDisplay(text), XtWindow(text), item_id);
		XmClipboardUnlock(XtDisplay(text), XtWindow(text), False);
		XtFree(szSelection);
		return;
	};

	/* End the copy and unlock the clipboard */
	XmClipboardEndCopy(XtDisplay(text), XtWindow(text), item_id);
	XmClipboardUnlock(XtDisplay(text), XtWindow(text), False);
}
