/*
	fileview.c
	Creates a Scrolled Text Widget that contains text loaded from a
	file or from standard input (if the filename is "-").

	mmore - A file/stream viewer for Motif/X
	Copyright (C) 2005   Matthew Holder (sixxgate@hotmail.com)

	derived from:
	mas - A file/stream viewer for X
	Copyright (C) 1998   Roberto Machorro (rmach@bigfoot.com)
	
	This program is free software; you can redistribute it and/or modify
	it under the terms of the GNU General Public License as published by
	the Free Software Foundation; either version 2 of the License, or
	(at your option) any later version.

	This program is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with this program; if not, write to the Free Software
	Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
*/

#include <Xm/Text.h>
#include "fileview.h"
#include "errdlg.h"

#define MAX_LINE_CHARS		2048
#define MAX_LINE_CHARS_DWORD	512

char RemoveRoffFormattings(char *szString);
char *szTranslations = "\
<Key>Q:		quit()\n\
<Key>space:	next-page()\n\
<Key>F:		next-page()\n\
Ctrl<Key>B:	previous-page()\n\
<Key>B:		previous-page()\n\
<Key>K:		scroll-one-line-down()\n\
<Key>Y:		scroll-one-line-down()\n\
<Key>Return:	scroll-one-line-up()\n\
<Key>J:		scroll-one-line-up()\n\
<Key>E:		scroll-one-line-up()\n\
Ctrl<Key>X:	copy-clipboard()\n\
Ctrl<Key>C:	copy-clipboard()\n\
Shift<Btn4Down>,<Btn4Up>:	scroll-one-line-down()\n\
Shift<Btn5Down>,<Btn5Up>:	scroll-one-line-up()\n\
Ctrl<Btn4Down>,<Btn4Up>:	previous-page()\n\
Ctrl<Btn5Down>,<Btn5Up>:	next-page()\n\
<Btn4Down>,<Btn4Up>:		scroll-one-line-down()scroll-one-line-down()scroll-one-line-down()\n\
<Btn5Down>,<Btn5Up>:		scroll-one-line-up()scroll-one-line-up()scroll-one-line-up()\n";
XtTranslations transTable = NULL;

void loadFile(Widget parent, Widget textView, char *filename)
{
	FILE *fin;
	char line[MAX_LINE_CHARS];
	XmTextPosition curpos;

	if (strcmp("-", filename))
		fin = fopen(filename, "r");
	else
		fin = stdin;

	if (fin == NULL) {
		errorDialog(parent, "Error opening input file.");
		return;
	}

	while (fgets(line, MAX_LINE_CHARS, fin) != NULL) {
		curpos = XmTextGetLastPosition(textView);
		RemoveRoffFormattings(line);
		XmTextInsert(textView, curpos, line);
	}

	XmTextSetInsertionPosition (textView, 0);

	fclose(fin);
}

Widget CreateFileViewer(Widget parent, Widget bottomWidget, char *filename)
{
	Cardinal n;
	Arg args[10];
	Widget textView;

	XtTranslations transTable = XtParseTranslationTable(szTranslations);
	
	n = 0;
	XtSetArg(args[n], XmNeditMode, XmMULTI_LINE_EDIT); n++;
	XtSetArg(args[n], XmNeditable, FALSE); n++;
	XtSetArg(args[n], XmNrows, 25); n++;
	XtSetArg(args[n], XmNcolumns, 80); n++;
	XtSetArg(args[n], XmNtopAttachment, XmATTACH_FORM); n++;
	XtSetArg(args[n], XmNleftAttachment, XmATTACH_FORM); n++;
	XtSetArg(args[n], XmNrightAttachment, XmATTACH_FORM); n++;
	XtSetArg(args[n], XmNbottomAttachment, XmATTACH_WIDGET); n++;
	XtSetArg(args[n], XmNbottomWidget, bottomWidget); n++;
	/* Setup less like key bindings, scroll wheel, and Windows like copy key bindings */
	textView = XmCreateScrolledText(parent, "FileView", args, n);

	XtOverrideTranslations(textView, transTable);

	loadFile(parent, textView, filename);

	return textView;
}

/**
 * Reformats a string by literally honoring the backspace characters.
 *
 * @param (in, out) The string to be reformatted.
 * @return 1 on success, 0 if szString is empty.
 */
char RemoveRoffFormattings(char *szString)
{
	char *pGet = szString;			/* szString test/get position */
	char *pSet = szString;			/* szString set position */
	int nLen = strlen(szString);		/* Length of szString */
	char *pEOS = szString + nLen + 1;	/* End of string location */
	
	/* Don't bother if the string is empty or just a new-line */
	if (nLen < 2) return 0;

	/* Skip the first character if it is a backspace */
	if (*pGet == '\x8') pGet++;

	for (; pGet < pEOS; pGet++)
	{
	/* Move back one character in the new string if we find a <bs>. */
		if (*pGet == '\x8')
			pSet--;
	/* Otherwise, just set the character and move to the next one. */
		else
			{ *pSet = *pGet; pSet++; }
	}

	return 1;
}
