/*
	mmore - A file/stream viewer for Motif/X
	Copyright (C) 2005   Matthew Holder (sixxgate@hotmail.com)

	derived from:
	mas - A file/stream viewer for X
	Copyright (C) 1998   Roberto Machorro (rmach@bigfoot.com)

	This program is free software; you can redistribute it and/or modify
	it under the terms of the GNU General Public License as published by
	the Free Software Foundation; either version 2 of the License, or
	(at your option) any later version.

	This program is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with this program; if not, write to the Free Software
	Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
*/

#include <Xm/Xm.h>
#include <Xm/Form.h>
#include <Xm/PushB.h>
#include <Xm/MwmUtil.h>
#include <X11/Xmu/Editres.h>
#include <stdio.h> /* sprintf */
#include "fileview.h"
#include "errdlg.h"
#include "textsearch.h"
#include "copytext.h"
#include "about.h"

void quit();
void quitAP();
void no_opAP();

XtActionsRec NewActions[] = {
    {"quit", quitAP}
};

char *translations = "<Key>q:		quit()\n";
XtTranslations transtable = NULL;

int main(int argc, char *argv[])
{
	Widget wndMMore, frmLayout, txtOutput;
	Widget btnSearch, btnAgain, btnCopy, btnQuit, btnHelp;
	XtAppContext xtAppContext;
	Arg args[5]; Cardinal n;
	XmString szGeneric;

	wndMMore = XtVaAppInitialize(&xtAppContext,
				   "MMore",
				   (XrmOptionDescList)NULL, 0,
				   &argc, argv,
				   (String*)NULL,
				   XmNkeyboardFocusPolicy, XmEXPLICIT,
				   XmNmwmDecorations, MWM_DECOR_ALL,
				   XmNtraversalOn, True,
				   XmNsensitive, True,
				   NULL);

	/* EditRes Support */
	XtAddEventHandler(wndMMore, (EventMask) 0, True,
              (XtEventHandler) _XEditResCheckMessages, NULL);

	/* Create the universal Quit key binding */
	XtTranslations transtable = XtParseTranslationTable(translations);

	n = 0;
	XtSetArg(args[n], XmNmarginHeight, 7); n++;
	XtSetArg(args[n], XmNmarginWidth, 7); n++;
	XtSetArg(args[n], XmNverticalSpacing, 3); n++;
	XtSetArg(args[n], XmNhorizontalSpacing, 3); n++;
	frmLayout = XmCreateForm(wndMMore, "Layout", args, n);

	n = 0;
	szGeneric = XmStringCreateLtoR("Find...", XmSTRING_DEFAULT_CHARSET);
	XtSetArg(args[n], XmNlabelString, szGeneric); n++;
	XtSetArg(args[n], XmNbottomAttachment, XmATTACH_FORM); n++;
	XtSetArg(args[n], XmNleftAttachment, XmATTACH_FORM); n++;
	XtSetArg(args[n], XmNbottomAttachment, XmATTACH_FORM); n++;
	XtSetArg(args[n], XmNmarginWidth, 5); n++;
	XtSetArg(args[n], XmNmarginHeight, 5); n++;
	btnSearch = XmCreatePushButton(frmLayout, "Search", args, n);
	XmStringFree(szGeneric);

	n = 0;
	szGeneric = XmStringCreateLtoR("Find Again", XmSTRING_DEFAULT_CHARSET);
	XtSetArg(args[n], XmNlabelString, szGeneric); n++;
	XtSetArg(args[n], XmNbottomAttachment, XmATTACH_FORM); n++;
	XtSetArg(args[n], XmNleftAttachment, XmATTACH_WIDGET); n++;
	XtSetArg(args[n], XmNleftWidget, btnSearch); n++;
	XtSetArg(args[n], XmNbottomAttachment, XmATTACH_FORM); n++;
	XtSetArg(args[n], XmNmarginWidth, 5); n++;
	XtSetArg(args[n], XmNmarginHeight, 5); n++;
	btnAgain = XmCreatePushButton(frmLayout, "Again", args, n);
	XmStringFree(szGeneric);

	n = 0;
	szGeneric = XmStringCreateLtoR("Copy", XmSTRING_DEFAULT_CHARSET);
	XtSetArg(args[n], XmNlabelString, szGeneric); n++;
	XtSetArg(args[n], XmNbottomAttachment, XmATTACH_FORM); n++;
	XtSetArg(args[n], XmNleftAttachment, XmATTACH_WIDGET); n++;
	XtSetArg(args[n], XmNleftWidget, btnAgain); n++;
	XtSetArg(args[n], XmNbottomAttachment, XmATTACH_FORM); n++;
	XtSetArg(args[n], XmNmarginWidth, 5); n++;
	XtSetArg(args[n], XmNmarginHeight, 5); n++;
	btnCopy = XmCreatePushButton(frmLayout, "Copy", args, n);
	XmStringFree(szGeneric);

	n = 0;
	szGeneric = XmStringCreateLtoR("Quit", XmSTRING_DEFAULT_CHARSET);
	XtSetArg(args[n], XmNlabelString, szGeneric); n++;
	XtSetArg(args[n], XmNbottomAttachment, XmATTACH_FORM); n++;
	XtSetArg(args[n], XmNleftAttachment, XmATTACH_WIDGET); n++;
	XtSetArg(args[n], XmNleftWidget, btnCopy); n++;
	XtSetArg(args[n], XmNbottomAttachment, XmATTACH_FORM); n++;
	XtSetArg(args[n], XmNmarginWidth, 5); n++;
	XtSetArg(args[n], XmNmarginHeight, 5); n++;
	btnQuit = XmCreatePushButton(frmLayout, "Quit", args, n);
	XmStringFree(szGeneric);

	n = 0;
	szGeneric = XmStringCreateLtoR("About", XmSTRING_DEFAULT_CHARSET);
	XtSetArg(args[n], XmNlabelString, szGeneric); n++;
	XtSetArg(args[n], XmNbottomAttachment, XmATTACH_FORM); n++;
	XtSetArg(args[n], XmNrightAttachment, XmATTACH_FORM); n++;
	XtSetArg(args[n], XmNbottomAttachment, XmATTACH_FORM); n++;
	XtSetArg(args[n], XmNmarginWidth, 5); n++;
	XtSetArg(args[n], XmNmarginHeight, 5); n++;
	btnHelp = XmCreatePushButton(frmLayout, "Help", args, n);
	XmStringFree(szGeneric);

	if (argc > 1)
		txtOutput = CreateFileViewer(frmLayout, btnSearch, argv[1]);
	else
		txtOutput = CreateFileViewer(frmLayout, btnSearch, "-");

	XtAddCallback(btnSearch, XmNactivateCallback, search, txtOutput);
	XtAddCallback(btnAgain, XmNactivateCallback, sagain, txtOutput);
	XtAddCallback(btnCopy, XmNactivateCallback, copy, txtOutput);
	XtAddCallback(btnQuit, XmNactivateCallback, quit, NULL);
	XtAddCallback(btnHelp, XmNactivateCallback, help, wndMMore);

	XtAppAddActions(xtAppContext, NewActions, XtNumber(NewActions));

	XtOverrideTranslations(btnHelp, transtable);
	XtOverrideTranslations(btnQuit, transtable);
	XtOverrideTranslations(btnCopy, transtable);
	XtOverrideTranslations(btnAgain, transtable);
	XtOverrideTranslations(btnSearch, transtable);
	XtOverrideTranslations(frmLayout, transtable);
	
	XtManageChild(txtOutput);
	XtManageChild(btnHelp);
	XtManageChild(btnQuit);
	XtManageChild(btnCopy);
	XtManageChild(btnAgain);
	XtManageChild(btnSearch);
	XtManageChild(frmLayout);

	XtRealizeWidget(wndMMore);

	/* Move the input focus to the text widget */
	XmProcessTraversal(txtOutput, XmTRAVERSE_CURRENT);

	XtAppMainLoop(xtAppContext);

	return 0;
}

void quit(Widget w, XtPointer client_data, XmAnyCallbackStruct *call)
{
	XtCloseDisplay(XtDisplay(w));
	exit(0);
}

void quitAP(Widget w, XEvent *event, String *args, Cardinal *nArgs)
{
	XtCloseDisplay(XtDisplay(w));
	exit(0);
}
