/*
	textsearch.c
	Searches an XmText widget for a string and hilights it.

	mmore - A file/stream viewer for Motif/X
	Copyright (C) 2005   Matthew Holder (sixxgate@hotmail.com)

	derived from:
	mas - A file/stream viewer for X
	Copyright (C) 1998   Roberto Machorro (rmach@bigfoot.com)

	This program is free software; you can redistribute it and/or modify
	it under the terms of the GNU General Public License as published by
	the Free Software Foundation; either version 2 of the License, or
	(at your option) any later version.

	This program is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with this program; if not, write to the Free Software
	Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
*/

#include <Xm/Form.h>
#include <Xm/PushB.h>
#include <Xm/Separator.h>
#include <Xm/Text.h>
#include <Xm/TextF.h>
#include <Xm/Label.h>
#include "errdlg.h"
#include "textsearch.h"

char *sstr = NULL;
Widget dlg, input;

void newsstr();

void search(Widget w, XtPointer client_data, XmAnyCallbackStruct *call)
{
	XmString message;
	Arg args[10]; Cardinal n;
	Widget label, divider, okbtn;
	Widget parent = (Widget)client_data;

	n = 0;
	XtSetArg(args[n], XmNtitle, "Find..."); n++;
	XtSetArg(args[n], XmNdialogStyle, XmDIALOG_FULL_APPLICATION_MODAL);n++;
	XtSetArg(args[n], XmNmarginHeight, 7); n++;
	XtSetArg(args[n], XmNmarginWidth, 7); n++;
	XtSetArg(args[n], XmNverticalSpacing, 3); n++;
	XtSetArg(args[n], XmNhorizontalSpacing, 3); n++;
	dlg = XmCreateFormDialog(parent, "SearchDialog", args, n);

	n = 0;
	message = XmStringCreateSimple("Search for:");
	XtSetArg(args[n], XmNalignment, XmALIGNMENT_BEGINNING); n++;
	XtSetArg(args[n], XmNlabelString, message); n++;
	XtSetArg(args[n], XmNtopAttachment, XmATTACH_FORM); n++;
	XtSetArg(args[n], XmNleftAttachment, XmATTACH_FORM); n++;
	XtSetArg(args[n], XmNrightAttachment, XmATTACH_FORM); n++;
	label = XmCreateLabel(dlg, "SearchLabel", args, n);
	XmStringFree(message);

	n = 0;
	XtSetArg(args[n], XmNcolumns, 64); n++;
	XtSetArg(args[n], XmNtopAttachment, XmATTACH_WIDGET); n++;
	XtSetArg(args[n], XmNtopWidget, label); n++;
	XtSetArg(args[n], XmNleftAttachment, XmATTACH_FORM); n++;
	XtSetArg(args[n], XmNrightAttachment, XmATTACH_FORM); n++;
	input = XmCreateTextField(dlg, "SearchInput", args, n);

	n = 0;
	XtSetArg(args[n], XmNtopAttachment, XmATTACH_WIDGET); n++;
	XtSetArg(args[n], XmNtopWidget, input); n++;
	XtSetArg(args[n], XmNleftAttachment, XmATTACH_FORM); n++;
	XtSetArg(args[n], XmNrightAttachment, XmATTACH_FORM); n++;
	divider = XmCreateSeparator(dlg, "Divider", args, n);

	n = 0;
	message = XmStringCreateSimple("OK");
	XtSetArg(args[n], XmNlabelString, message); n++;
	XtSetArg(args[n], XmNshowAsDefault, True); n++;
	XtSetArg(args[n], XmNtopAttachment, XmATTACH_WIDGET); n++;
	XtSetArg(args[n], XmNtopWidget, divider); n++;
	XtSetArg(args[n], XmNrightAttachment, XmATTACH_FORM); n++;
	XtSetArg(args[n], XmNbottomAttachment, XmATTACH_FORM); n++;
	okbtn = XmCreatePushButton(dlg, "OKBTN", args, n);
	XmStringFree(message);

	XtAddCallback(okbtn, XmNactivateCallback, newsstr, client_data);
	XtAddCallback(input, XmNactivateCallback, newsstr, client_data);

	XtManageChild(okbtn);
	XtManageChild(divider);
	XtManageChild(input);
	XtManageChild(label);
	XtManageChild(dlg);
	
	XmProcessTraversal(input, XmTRAVERSE_CURRENT);
}

void sagain(Widget w, XtPointer client_data, XmAnyCallbackStruct *call)
{
	if (sstr == NULL)
		search(w, client_data, call);
	else {
		int tamsstr = strlen(sstr);
		Widget parent = (Widget)client_data;
		XmTextPosition pos = XmTextGetCursorPosition(parent);
		XButtonEvent *p_event = (XButtonEvent *)call->event;
		if (XmTextFindString(parent, pos, sstr, XmTEXT_FORWARD, &pos))
		{
			XmTextShowPosition(parent, pos);
			XmTextSetCursorPosition(parent, pos + tamsstr);
			XmTextSetSelection(parent, pos, pos + tamsstr,
					   p_event->time);
		} else
			errorDialog((Widget)client_data, "String not found.");
	}
}

void newsstr(Widget w, XtPointer client_data, XmAnyCallbackStruct *call)
{
	if (sstr != NULL) XtFree(sstr);
	sstr = XmTextFieldGetString(input);
	sagain(w, client_data, call);
	XtDestroyWidget(dlg);
}
